@@ -1,18 +1,28 @@
+# SPDX-FileCopyrightText: © 2025 DSLab - Fondazione Bruno Kessler
+#
+# SPDX-License-Identifier: Apache-2.0
+
 from __future__ import annotations
 
 import pickle
+import typing
 from typing import Any
 
-from digitalhub_core.entities._base.state import State
-from digitalhub_core.entities.artifact.crud import log_artifact
-from digitalhub_core.entities.artifact.entity._base import Artifact
-from digitalhub_core.utils.logger import LOGGER
-from digitalhub_data.entities.dataitem.crud import log_dataitem
-from digitalhub_data.entities.dataitem.entity.table import DataitemTable
-from digitalhub_data.readers.registry import DATAFRAME_TYPES
+from digitalhub.entities._commons.enums import EntityKinds, Relationship, State
+from digitalhub.entities.artifact._base.entity import Artifact
+from digitalhub.entities.artifact.crud import log_artifact
+from digitalhub.entities.dataitem._base.entity import Dataitem
+from digitalhub.entities.dataitem.crud import log_dataitem
+from digitalhub.entities.model._base.entity import Model
+from digitalhub.stores.readers.data.api import get_supported_dataframes
+from digitalhub.utils.exceptions import EntityNotExistsError
+from digitalhub.utils.logger import LOGGER
+
+if typing.TYPE_CHECKING:
+    from digitalhub.entities.dataitem.table.entity import DataitemTable
 
 
-def collect_outputs(results: Any, outputs: list[str], project_name: str) -> dict:
+def collect_outputs(results: Any, outputs: list[str], project_name: str, run_key: str) -> dict:
     """
     Collect outputs. Use the produced results directly.
 
@@ -20,8 +30,12 @@ def collect_outputs(results: Any, outputs: list[str], project_name: str) -> dict
     ----------
     results : Any
         Function outputs.
-    project : Project
-        Project object.
+    outputs : list[str]
+        List of named outputs to collect.
+    project_name : str
+        Project name.
+    run_key : str
+        Run key.
 
     Returns
     -------
@@ -32,6 +46,7 @@ def collect_outputs(results: Any, outputs: list[str], project_name: str) -> dict
     results = listify_results(results)
 
     for idx, item in enumerate(results):
+        # Get mapping of outputs, if not found, create a new one
         try:
             name = outputs[idx]
         except IndexError:
@@ -40,16 +55,34 @@ def collect_outputs(results: Any, outputs: list[str], project_name: str) -> dict
         if isinstance(item, (str, int, float, bool, bytes)):
             objects[name] = item
 
-        elif f"{item.__class__.__module__}.{item.__class__.__name__}" in DATAFRAME_TYPES:
-            objects[name] = _log_dataitem(name, project_name, item)
-
         else:
-            objects[name] = _log_artifact(name, project_name, item)
+            # Recieve a dataframe object
+            if f"{item.__class__.__module__}.{item.__class__.__name__}" in get_supported_dataframes():
+                obj = _log_dataitem(name, project_name, item)
+
+            # Recieve a digitalhub object
+            elif isinstance(item, (Dataitem, Artifact, Model)):
+                obj = item
+
+                # Add relationship to object, update it
+                dest = run_key + ":" + run_key.split("/")[-1]
+                obj.add_relationship(relation=Relationship.PRODUCEDBY.value, dest=dest)
+
+                try:
+                    obj.save(update=True)
+                except EntityNotExistsError:
+                    obj.save()
+
+            # Recieve a generic python object
+            else:
+                obj = _log_artifact(name, project_name, item)
+
+            objects[name] = obj
 
     return objects
 
 
-def parse_outputs(results: Any, run_outputs: list, project_name: str) -> dict:
+def parse_outputs(results: Any, run_outputs: list, project_name: str, run_key: str) -> dict:
     """
     Parse outputs.
 
@@ -57,8 +90,12 @@ def parse_outputs(results: Any, run_outputs: list, project_name: str) -> dict:
     ----------
     results : Any
         Function outputs.
-    project : Project
-        Project object.
+    run_outputs : list
+        Run outputs.
+    project_name : str
+        Project name.
+    run_key : str
+        Run key.
 
     Returns
     -------
@@ -72,7 +109,7 @@ def parse_outputs(results: Any, run_outputs: list, project_name: str) -> dict:
             out_list.append(run_outputs.pop(0))
         except IndexError:
             out_list.append(f"output_{idx}")
-    return collect_outputs(results, out_list, project_name)
+    return collect_outputs(results, out_list, project_name, run_key)
 
 
 def listify_results(results: Any) -> list:
@@ -109,7 +146,7 @@ def _log_dataitem(name: str, project_name: str, data: Any) -> DataitemTable:
     project_name : str
         Project name.
     data : Any
-        Dataframe.
+        Data.
 
     Returns
     -------
@@ -120,7 +157,7 @@ def _log_dataitem(name: str, project_name: str, data: Any) -> DataitemTable:
         return log_dataitem(
             project=project_name,
             name=name,
-            kind="table",
+            kind=EntityKinds.DATAITEM_TABLE.value,
             data=data,
         )
     except Exception as e:
@@ -155,7 +192,7 @@ def _log_artifact(name: str, project_name: str, data: Any) -> Artifact:
         return log_artifact(
             project=project_name,
             name=name,
-            kind="artifact",
+            kind=EntityKinds.ARTIFACT_ARTIFACT.value,
             source=pickle_file,
         )
 
@@ -191,7 +228,7 @@ def build_status(
 
     try:
         for key, value in parsed_execution.items():
-            if isinstance(value, (DataitemTable, Artifact)):
+            if isinstance(value, (Dataitem, Artifact, Model)):
                 outputs[key] = value.key
             else:
                 results[key] = value
